#!/bin/sh -e
# SPDX-License-Identifier: MIT

SETTING_NAME="$1"
AUTH_TYPE="$2"
VPN_CONF_FILE="$3"

CONF_DIR="/etc/atmark/abos_web/openvpn"
USERPASS_FILE="userpass.txt"
KEY_PASS_FILE="keypass.txt"
VPN_INFO_FILE="vpn_info.txt"
CONTAINER_IMAGE_NAME="alpine_openvpn"
CONTAINER_NAME="abos_web_openvpn"
CONTAINER_CONF_FILE="/etc/atmark/containers/abos_web_openvpn.conf"
UP_SCRIPT="/usr/libexec/abos-web/openvpn/up.sh"
DOWN_SCRIPT="/usr/libexec/abos-web/openvpn/down.sh"
DNS_FILE="/etc/dnsmasq.d/openvpn_dns.conf"

trap catch ERR

catch() {
	if podman container exists "$CONTAINER"; then
		podman stop "$CONTAINER" || :
		podman rm "$CONTAINER" || :
	fi
	rm -rf "$CONF_DIR"
	persist_file -d -r "$CONF_DIR" || :
	rm -f "$CONTAINER_CONF_FILE"
	persist_file -d "$CONTAINER_CONF_FILE" || :
	rm -f "$DNS_FILE"
	persist_file -d "$DNS_FILE" || :
	rc-service dnsmasq restart
}

check_filename() {
	case "$1" in
		*[!a-zA-Z0-9=_.,:-]*)
			echo "Invalid file name: $1" >&2
			exit 1 ;;
	esac
}

check_filename "$VPN_CONF_FILE"

while [ $# -gt 0 ];
do
	case "$1" in
		"--user")
			shift
			USER="$1" ;;
		"--password")
			shift
			PASSWORD="$1" ;;
		"--cert")
			shift
			CERT_FILE="$1"
			check_filename "$CERT_FILE" ;;
		"--key")
			shift
			KEY_FILE="$1"
			check_filename "$KEY_FILE";;
		"--askpass")
			shift
			KEY_PASS="$1" ;;
		*)
			;;
	esac
	shift
done

podman stop -a
# wait for containers to be fully stopped
podman ps --format '{{.ID}}' \
	| timeout 20s xargs -r podman wait > /dev/null
abos-ctrl podman-rw build -t "$CONTAINER_IMAGE_NAME":latest - << 'EOF'
From docker.io/alpine:latest
RUN apk add --no-cache openvpn
EOF
podman_start -a

opt=""
if [ "$AUTH_TYPE" = "userpass" ]; then
	cat > "$CONF_DIR/$USERPASS_FILE" <<EOF
$USER
$PASSWORD
EOF
	chmod 400 "$CONF_DIR/$USERPASS_FILE"
	opt="$opt --auth-user-pass $USERPASS_FILE"
else
	if [ -n "$CERT_FILE" ]; then
		opt="$opt --cert $CERT_FILE"
	fi
	if [ -n "$KEY_FILE" ]; then
		opt="$opt --key $KEY_FILE"
	fi
	if [ -n "$KEY_PASS" ]; then
		cat > "$CONF_DIR/$KEY_PASS_FILE" <<EOF
$KEY_PASS
EOF
		chmod 400 "$CONF_DIR/$KEY_PASS_FILE"
		opt="$opt --askpass $KEY_PASS_FILE"
	fi
fi

if grep -q "^cipher AES-128-CBC" "$CONF_DIR/$VPN_CONF_FILE"; then
	opt="$opt --data-ciphers AES-128-CBC"
fi

cat > "$DNS_FILE" <<EOF
# resolv-file's directory must exist when dnsmasq starts. We already create
# /run/NetworkMamanger for normal dns operations, so use the same directory
resolv-file=/run/NetworkManager/openvpn_resolv.conf
strict-order
EOF

opt="$opt --script-security 2 --up $UP_SCRIPT --down $DOWN_SCRIPT"

cat > "$CONTAINER_CONF_FILE" <<EOF
set_image alpine_openvpn
set_network host
add_devices /dev/net/tun
add_volumes /etc/atmark/abos_web/openvpn
add_volumes /run/NetworkManager
add_volumes /usr/libexec/abos-web/openvpn
add_args -w /etc/atmark/abos_web/openvpn
add_args --cap-add=NET_ADMIN

# replace with a more useful command
set_command openvpn --config $VPN_CONF_FILE $opt
EOF

cat > "$CONF_DIR/$VPN_INFO_FILE" <<EOF
$SETTING_NAME
EOF

rc-service dnsmasq restart
podman_start "$CONTAINER_NAME"
persist_file -r "$CONF_DIR"
persist_file "$CONTAINER_CONF_FILE"
persist_file "$DNS_FILE"
"$(dirname "$0")/podman.sh" logs_follow "$CONTAINER_NAME"
